//*****************************************************************************
//
// qtmixer - an X windows soundcard mixer program.
// Copyright (C) 1997 - 2000  Tom Daley
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// The license is in the file COPYING.
//
// Tom Daley
// 1309 W. Lorraine Ave.
// Woodland Park, CO  80863-2329
//
// daley@codenet.net (Tom Daley)
// http://users.codenet.net/daley
//
//*****************************************************************************

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/soundcard.h>

#include <qbitmap.h>

#include "mixer.h"
#include "qtmixer.xbm"

const char *QtedMixer::labels[] = SOUND_DEVICE_LABELS;

//*****************************************************************************

QtedMixer::
QtedMixer(QIntDict<QString> channel_titles, QString mixer_name,
	  QWidget *parent, const char *name) : QWidget(parent, name) {
   QBitmap icon(qtmixer_width, qtmixer_height, qtmixer_bits, TRUE);
   setIcon(icon);

   _valid = 1;
   _fd = open(mixer_name, O_RDWR);
   if (_fd < 0) {
      QString error_msg;
      _valid = 0;
      error_msg.sprintf("Can't open %s", (const char *)mixer_name);
      error(error_msg); 
   }
  
   if (valid()) {
      int mask;
      if (ioctl(_fd, SOUND_MIXER_READ_CAPS, &mask) == -1) {
         _valid = 0;
         error("Read capabilities", "No Mixer available");
      } else
         _exclusive_record = mask & SOUND_CAP_EXCL_INPUT;
   }

   if (valid()) {
      if (ioctl(_fd, SOUND_MIXER_READ_DEVMASK, &_channel_bits)  == -1)
         error("Read available devices");

      if (ioctl(_fd, SOUND_MIXER_READ_RECMASK, &_recordable_bits)  == -1)
         error("Read available recording devices");

      if (ioctl(_fd, SOUND_MIXER_READ_RECSRC, &_record_bits)  == -1)
         error("Read recording sources");

      if (ioctl(_fd, SOUND_MIXER_READ_STEREODEVS, &_stereo_bits)  == -1)
         error("Read stereo capability");

      int min_slider_width = 0;
      int channel = 0;
      for (int i = 0; i < SOUND_MIXER_NRDEVICES; i++) {
         if (_channel_bits & (1 << i)) {
            int vol, right, left;
            if (ioctl(_fd, MIXER_READ(i), &vol)==-1) {
               error ("sound mixer read", "undefined channel");
            } else {
               left = vol & 0xFF;
               right = (vol >> 8) & 0xFF; 
            }
            QtedMixerChannel *channel_p;
            QString title(labels[i]);
            if (channel_titles.find(channel))
               title = *(channel_titles.find(channel));

            channel_p = new QtedMixerChannel(i, title, left, right, 
                                             _recordable_bits & (1 << i), this);
            channel++;
            if (!(_stereo_bits & (1 << i)))
               channel_p->setMono(1);
   
            if (_record_bits & (1 << i))
               channel_p->setRecord(1);
        
            _channel_list.append(channel_p);
            connect(channel_p, SIGNAL(valueChanged(QtedMixerChannel*)),
                    this, SLOT(setValue(QtedMixerChannel*)));
            connect(channel_p, SIGNAL(recordPressed(QtedMixerChannel*)),
                    this, SLOT(setRecord(QtedMixerChannel*)));

            if (fontMetrics().width(labels[i]) > min_slider_width)
               min_slider_width = fontMetrics().width(labels[i]); 
         }
      }

      int space = 4;
      setMinimumSize((min_slider_width + space) * _channel_list.count(), 200);

      resizeEvent(NULL);
   }
}

//*****************************************************************************

void QtedMixer::
resizeEvent(QResizeEvent *) {
   int x = 0, y = 0;
   int count = _channel_list.count();

   for (int i = 0; i < count; i++) {
      _channel_list.at(i)->setGeometry(x, y, width()/count, height());
      x += width()/count;
   }
}

//*****************************************************************************

int QtedMixer::
valid(void) {
   return (_valid);
}

//*****************************************************************************

void QtedMixer::
error(QString label, QString reason) {
   if (errno == ENXIO)
      _error_str.sprintf("%s, %s", (const char *)label, (const char *)reason);
   else
      _error_str.sprintf("%s, %s", (const char *)label, strerror(errno));
   emit error(&_error_str);
}

//*****************************************************************************

QString QtedMixer::
errorString(void) {
   return (_error_str);
}

//*****************************************************************************

void QtedMixer::
setValue(QtedMixerChannel *ch_p) {
   int level;
   level = (ch_p->rightValue() & 0xFF) << 8;
   level += (ch_p->leftValue() & 0xFF);
   if (ioctl(_fd, MIXER_WRITE(ch_p->number()), &level) == -1)
      error("Write to mixer");
}

//*****************************************************************************

void QtedMixer::
setRecord(QtedMixerChannel *ch_p) {
   if (ch_p->recordOn() == 0) {	// turn off recording for this channel
      _record_bits &= ~(1 << ch_p->number());
      if (ioctl(_fd, SOUND_MIXER_WRITE_RECSRC, &_record_bits)  == -1)
         error("Read recording sources");
   } else {			// turn on recording
      if (_exclusive_record) {	// if exclusive, turn off any enabled channels
         for (int i = 0; i < _channel_list.count(); i++) {
            if (_record_bits & (1 << i)) {
               _channel_list.at(i)->setRecord(0);
            }
         }
         _record_bits = 0;
      }
      _record_bits |= (1 << ch_p->number());
      if (ioctl(_fd, SOUND_MIXER_WRITE_RECSRC, &_record_bits)  == -1)
         error("Read recording sources");
   }
}

//*****************************************************************************
